#! /usr/bin/perl
use Getopt::Long;
use Time::Local;
$Getopt::Long::ignorecase=undef;
GetOptions('t=s','n=s','l=s');

#---------------------------------
# Set Parameters
#---------------------------------
$NHOUR = 6;   # Number for learning hour
$LBASE = 10;  # Learning parameter
$PLATFORM_DATA_MNG = "./PLATFORM_DATA_MNG.prm";
$PLATFORM_REAL_MNG = "../exec_platform_mng/ZEXEC_REAL_FST_MNG.prm";
#---------------------------------

if ( length($opt_t)!=10 ) {
   $YMDH  = `date +%y%m%d%H`;
   chomp($YMDH);
   $YMDHM = sprintf "%s00",${YMDH};
} else {
   $YMDHM = $opt_t;
}
print "=== YMDH= $YMDHM, NHOUR= $NHOUR, LBASE= $LBASE\n";

#---------------------------------
# Read Parameters
#---------------------------------
system "rm -f shudan_*.prm";
open(IN,"<$PLATFORM_REAL_MNG");
$flag = 0;
while (<IN>) {
 chomp($line=$_);
 if ( $line =~ /^\<\/SHUUDAN\>/ ) { $flag=0; }
 if ( $flag == 1 ) {
   ($Module_code,$Area_code,$WF_code,$Tab_PATH)=split(/,/,$line);
   $fname = sprintf "shudan_%02d_%03d.prm",$Area_code,$WF_code;
   push(@FNPARM,$fname);
   #print "$fname\n";
   open(OUT,">>$fname");
   print OUT "$line\n";
   close(OUT);
 }
 if ( $line =~ /^\<SHUUDAN\>/ )   { $flag=1; }
}
close(IN);

open(IN,"<$PLATFORM_DATA_MNG");
while (<IN>) {
 chomp($line=$_);
 unless ( $line =~ /^#/ ) {
   ($Module_code,$Area_code,$WF_code,$Forecast_PATH,$OBS_PATH)=split(/,/,$line);
   $fname = sprintf "shudan_%02d_%03d.prm",$Area_code,$WF_code;
   #print "$fname\n";
   $flag = 0;
   foreach $ccc (@FNPARM) {
     #print "$fname ::: $ccc\n";
     if ( $fname eq $ccc ) {
        $flag = 1;
     }
   }
   if ( $flag == 1 ) {
     open(OUT,">>$fname");
     print OUT "$line\n";
     close(OUT);
   }
 }
}
close(IN);

#=========================================
# Loop for shudan_*.prm
#=========================================
foreach $parm (@FNPARM) {

#---------------------------------
# Open shudan_??_???.prm
#---------------------------------
open(IN,"<$parm");
chomp($line=<IN>);
($Module_code,$Area_code,$WF_code,$Tab_FILE)=split(/,/,$line);
require "$Tab_FILE";

$nfcst   = 0;
@PARMDATA=();
while(<IN>){
  #---------------------------------
  # Read shudan_??_???.prm
  #---------------------------------
  chomp($line=$_);
  push(@PARMDATA,$line);
  $nfcst++;
}
close(IN);

#---------------------------------
# !!!!!!!!!!! Read Beta info
#---------------------------------
@BETA=();@NBETA=();
$Beta_File = "./data/beta_${Area_code}_${WF_code}.out";
if ( -e $Beta_File ) {
   open(BBB,"<$Beta_File");
   while(<BBB>) {
      chomp($_);
      ($Module_code,$beta)=split(/,/,$_);
      push(@BETA,$beta);
   }
   close(BBB);
} else {
   for($i=0;$i<100;$i++) {
      push(@BETA,1.0);      
   } 
}

foreach $line (@PARMDATA) {

  #---------------------------------
  # Read shudan_??_???.prm
  #---------------------------------
  ($Module_code,$Area_code,$WF_code,$Forecast_PATH,$OBS_PATH)=split(/,/,$line);

  #---------------------------------
  # Make obs-fcst dataset
  #---------------------------------
  $cmd = "./mk_realfst.pl -h $NHOUR -f $Forecast_PATH -o $OBS_PATH -m $Module_code -a $Area_code -w $WF_code -p $POWER_MAX -t $YMDHM > log";
  print  "$cmd\n";
  system "$cmd\n";

  #---------------------------------
  # Evaluate error
  #---------------------------------
  $DSET_File = "./data/DSET_${Module_code}_${Area_code}_${WF_code}.out";
  open(IN2,"<$DSET_File");
  $nn  = 0;
  $mae = 0;
  while (<IN2>) {
     chomp($_);
     ($yy,$mm,$dd,$hh,$mn,$obs,$pobs,$fcs,$pfcs,@dummy)= split(/,/,$_);
     if ( $obs>0 && $fcs>0 ) {
        #print "$obs,$fcs\n";
        $err = abs($fcs-$obs);
        $mae = $mae + $err;
        $nn++;
     }
  }
  close(IN2);
  if ( $nn > 1 ) {
     push(@SCORE,$mae/$nn);
  } else {
     push(@SCORE,1e10);
  }

}

#---------------------------------
# Model Select
#---------------------------------
$Beta_File = "./data/beta_${Area_code}_${WF_code}.out";
open(OUT,">$Beta_File");
$num = 0;
$minscore = 1e10;
foreach $ss ( @SCORE ) {
   $num++;
   if ( $ss < $minscore ) {
      $minscore = $ss;
      $select   = $num;
   }
   printf OUT "%7.2f,",$ss;
}
printf OUT "$select\n";
close(OUT);

#---------------------------------
# Output
#---------------------------------
$num = 0;
foreach $line (@PARMDATA) {

  $num++;
  if ( $select == $num ) {
     ($Module_code,$Area_code,$WF_code,$Forecast_PATH,$OBS_PATH)=split(/,/,$line);
     $FFILE = sprintf "%s/c%s/%sf%s_20%s.csv"
           ,$Forecast_PATH,substr($YMDHM,0,4),$Area_code,$WF_code,$YMDHM;
     $DIR_SHUDAN   = sprintf "%s/c%s",$FST_DIR,substr($YMDHM,0,4);
     $FFILE_SHUDAN = sprintf "%s/%sf%s_20%s.csv",$DIR_SHUDAN,$Area_code,$WF_code,$YMDHM;
     system "mkdir -p $DIR_SHUDAN";
     system "cp $FFILE $FFILE_SHUDAN";
  }

}

}
#=========================================
# Loop for shudan_*.prm
#=========================================


